@@ -32,12 +32,10 @@
 #include <string.h>
 #include <unistd.h>
 #include <locale.h>
+#include <gio/gio.h>
 #include <glib/gprintf.h>
 #include <glib/gi18n.h>
 
-#include <libxml/xpath.h>
-#include <libxml/uri.h>
-
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
@@ -50,11 +48,12 @@
 #include <windows.h>
 #endif
 
-#include "virt-gtk-compat.h"
 #include "virt-viewer-app.h"
+#include "virt-viewer-resources.h"
 #include "virt-viewer-auth.h"
 #include "virt-viewer-window.h"
 #include "virt-viewer-session.h"
+#include "virt-viewer-util.h"
 #ifdef HAVE_GTK_VNC
 #include "virt-viewer-session-vnc.h"
 #endif
@@ -103,6 +102,7 @@ static void virt_viewer_app_update_pretty_address(VirtViewerApp *self);
 static void virt_viewer_app_set_fullscreen(VirtViewerApp *self, gboolean fullscreen);
 static void virt_viewer_app_update_menu_displays(VirtViewerApp *self);
 static void virt_viewer_update_smartcard_accels(VirtViewerApp *self);
+static void virt_viewer_app_add_option_entries(VirtViewerApp *self, GOptionContext *context, GOptionGroup *group);
 
 
 struct _VirtViewerAppPrivate {
@@ -114,6 +114,7 @@ struct _VirtViewerAppPrivate {
     gchar *clipboard;
     GtkWidget *preferences;
     GtkFileChooser *preferences_shared_folder;
+    GResource *resource;
     gboolean direct;
     gboolean verbose;
     gboolean enable_accel;
@@ -155,7 +156,7 @@ struct _VirtViewerAppPrivate {
 };
 
 
-G_DEFINE_ABSTRACT_TYPE(VirtViewerApp, virt_viewer_app, G_TYPE_OBJECT)
+G_DEFINE_ABSTRACT_TYPE(VirtViewerApp, virt_viewer_app, GTK_TYPE_APPLICATION)
 #define GET_PRIVATE(o)                                                        \
     (G_TYPE_INSTANCE_GET_PRIVATE ((o), VIRT_VIEWER_TYPE_APP, VirtViewerAppPrivate))
 
@@ -174,18 +175,9 @@ enum {
     PROP_UUID,
 };
 
-enum {
-    SIGNAL_WINDOW_ADDED,
-    SIGNAL_WINDOW_REMOVED,
-    SIGNAL_LAST,
-};
-
-static guint signals[SIGNAL_LAST];
-
 void
 virt_viewer_app_set_debug(gboolean debug)
 {
-#if GLIB_CHECK_VERSION(2, 31, 0)
     if (debug) {
         const gchar *doms = g_getenv("G_MESSAGES_DEBUG");
         if (!doms) {
@@ -197,7 +189,6 @@ virt_viewer_app_set_debug(gboolean debug)
             g_free(newdoms);
         }
     }
-#endif
     doDebug = debug;
 }
 
@@ -259,7 +250,7 @@ virt_viewer_app_save_config(VirtViewerApp *self)
         g_warning("failed to create config directory");
     g_free(dir);
 
-    if (priv->uuid && priv->guest_name) {
+    if (priv->uuid && priv->guest_name && g_key_file_has_group(priv->config, priv->uuid)) {
         // if there's no comment for this uuid settings group, add a comment
         // with the vm name so user can make sense of it later.
         gchar *comment = g_key_file_get_comment(priv->config, priv->uuid, NULL, &error);
@@ -298,7 +289,7 @@ virt_viewer_app_quit(VirtViewerApp *self)
         }
     }
 
-    gtk_main_quit();
+    g_application_quit(G_APPLICATION(self));
 }
 
 static gint
@@ -371,79 +362,6 @@ app_window_try_fullscreen(VirtViewerApp *self G_GNUC_UNUSED,
     virt_viewer_window_enter_fullscreen(win, monitor);
 }
 
-
-static GHashTable*
-virt_viewer_app_parse_monitor_mappings(gchar **mappings, gsize nmappings)
-{
-    gint nmonitors = get_n_client_monitors();
-    GHashTable *displaymap = g_hash_table_new(g_direct_hash, g_direct_equal);
-    GHashTable *monitormap = g_hash_table_new(g_direct_hash, g_direct_equal);
-    int i = 0;
-    int max_display_id = 0;
-    gchar **tokens = NULL;
-
-    for (i = 0; i < nmappings; i++) {
-        gchar *endptr = NULL;
-        gint display = 0, monitor = 0;
-
-        tokens = g_strsplit(mappings[i], ":", 2);
-        if (g_strv_length(tokens) != 2) {
-            g_warning("Invalid monitor-mapping configuration: '%s'. "
-                      "Expected format is '<DISPLAY-ID>:<MONITOR-ID>'",
-                      mappings[i]);
-            g_strfreev(tokens);
-            goto configerror;
-        }
-
-        display = strtol(tokens[0], &endptr, 10);
-        if ((endptr && *endptr != '\0') || display < 1) {
-            g_warning("Invalid monitor-mapping configuration: display id is invalid: %s %p='%s'", tokens[0], endptr, endptr);
-            g_strfreev(tokens);
-            goto configerror;
-        }
-        monitor = strtol(tokens[1], &endptr, 10);
-        if ((endptr && *endptr != '\0') || monitor < 1) {
-            g_warning("Invalid monitor-mapping configuration: monitor id '%s' is invalid", tokens[1]);
-            g_strfreev(tokens);
-            goto configerror;
-        }
-        g_strfreev(tokens);
-
-        if (monitor > nmonitors)
-            g_warning("Initial monitor #%i for display #%i does not exist, skipping...", monitor, display);
-        else {
-            /* config file format is 1-based, not 0-based */
-            display--;
-            monitor--;
-
-            if (g_hash_table_lookup_extended(displaymap, GINT_TO_POINTER(display), NULL, NULL) ||
-                g_hash_table_lookup_extended(monitormap, GINT_TO_POINTER(monitor), NULL, NULL)) {
-                g_warning("Invalid monitor-mapping configuration: a display or monitor id was specified twice");
-                goto configerror;
-            }
-            g_debug("Fullscreen config: mapping guest display %i to monitor %i", display, monitor);
-            g_hash_table_insert(displaymap, GINT_TO_POINTER(display), GINT_TO_POINTER(monitor));
-            g_hash_table_insert(monitormap, GINT_TO_POINTER(monitor), GINT_TO_POINTER(display));
-            max_display_id = MAX(display, max_display_id);
-        }
-    }
-
-    for (i = 0; i < max_display_id; i++) {
-        if (!g_hash_table_lookup_extended(displaymap, GINT_TO_POINTER(i), NULL, NULL)) {
-            g_warning("Invalid monitor-mapping configuration: display #%d was not specified", i+1);
-            goto configerror;
-        }
-    }
-
-    g_hash_table_unref(monitormap);
-    return displaymap;
-
-configerror:
-    g_hash_table_unref(monitormap);
-    g_hash_table_unref(displaymap);
-    return NULL;
-}
-
 static GHashTable*
 virt_viewer_app_get_monitor_mapping_for_section(VirtViewerApp *self, const gchar *section)
 {
@@ -460,7 +378,7 @@ virt_viewer_app_get_monitor_mapping_for_section(VirtViewerApp *self, const gchar
             g_warning("Error reading monitor assignments for %s: %s", section, error->message);
         g_clear_error(&error);
     } else {
-        mapping = virt_viewer_app_parse_monitor_mappings(mappings, nmappings);
+        mapping = virt_viewer_parse_monitor_mappings(mappings, nmappings, get_n_client_monitors());
     }
     g_strfreev(mappings);
 
@@ -468,15 +386,15 @@ virt_viewer_app_get_monitor_mapping_for_section(VirtViewerApp *self, const gchar
 }
 
 static
-void virt_viewer_app_set_uuid_string(VirtViewerApp *self, const gchar *uuid_string)
+void virt_viewer_app_apply_monitor_mapping(VirtViewerApp *self)
 {
     GHashTable *mapping = NULL;
 
-    g_debug("%s: UUID changed to %s", G_STRFUNC, uuid_string);
+    // apply mapping only in fullscreen
+    if (!virt_viewer_app_get_fullscreen(self))
+        return;
 
-    g_free(self->priv->uuid);
-    self->priv->uuid = g_strdup(uuid_string);
-    mapping = virt_viewer_app_get_monitor_mapping_for_section(self, uuid_string);
+    mapping = virt_viewer_app_get_monitor_mapping_for_section(self, self->priv->uuid);
     if (!mapping) {
         g_debug("No guest-specific fullscreen config, using fallback");
         mapping = virt_viewer_app_get_monitor_mapping_for_section(self, "fallback");
@@ -489,7 +407,7 @@ void virt_viewer_app_set_uuid_string(VirtViewerApp *self, const gchar *uuid_stri
 
     // if we're changing our initial display map, move any existing windows to
     // the appropriate monitors according to the per-vm configuration
-    if (mapping && self->priv->fullscreen) {
+    if (mapping) {
         GList *l;
         gint i = 0;
 
@@ -500,6 +418,20 @@ void virt_viewer_app_set_uuid_string(VirtViewerApp *self, const gchar *uuid_stri
     }
 }
 
+static
+void virt_viewer_app_set_uuid_string(VirtViewerApp *self, const gchar *uuid_string)
+{
+    if (g_strcmp0(self->priv->uuid, uuid_string) == 0)
+        return;
+
+    g_debug("%s: UUID changed to %s", G_STRFUNC, uuid_string);
+
+    g_free(self->priv->uuid);
+    self->priv->uuid = g_strdup(uuid_string);
+
+    virt_viewer_app_apply_monitor_mapping(self);
+}
+
 void
 virt_viewer_app_maybe_quit(VirtViewerApp *self, VirtViewerWindow *window)
 {
@@ -596,7 +528,10 @@ static void hide_one_window(gpointer value,
                             gpointer user_data G_GNUC_UNUSED)
 {
     VirtViewerApp* self = VIRT_VIEWER_APP(user_data);
-    if (self->priv->main_window != value)
+    VirtViewerAppPrivate *priv = self->priv;
+    gboolean connect_error = !priv->connected && !priv->cancelled;
+
+    if (connect_error || self->priv->main_window != value)
         virt_viewer_window_hide(VIRT_VIEWER_WINDOW(value));
 }
 
@@ -903,6 +838,13 @@ viewer_window_focus_out_cb(GtkWindow *window G_GNUC_UNUSED,
     return FALSE;
 }
 
+static gboolean
+virt_viewer_app_has_usbredir(VirtViewerApp *self)
+{
+    return virt_viewer_app_has_session(self) &&
+           virt_viewer_session_get_has_usbredir(virt_viewer_app_get_session(self));
+}
+
 static VirtViewerWindow*
 virt_viewer_app_window_new(VirtViewerApp *self, gint nth)
 {
@@ -921,17 +863,15 @@ virt_viewer_app_window_new(VirtViewerApp *self, gint nth)
     self->priv->windows = g_list_append(self->priv->windows, window);
     virt_viewer_app_set_window_subtitle(self, window, nth);
     virt_viewer_app_update_menu_displays(self);
-    if (self->priv->session) {
-        virt_viewer_window_set_usb_options_sensitive(window,
-                    virt_viewer_session_get_has_usbredir(self->priv->session));
-    }
+    virt_viewer_window_set_usb_options_sensitive(window, virt_viewer_app_has_usbredir(self));
 
-    g_signal_emit(self, signals[SIGNAL_WINDOW_ADDED], 0, window);
+    w = virt_viewer_window_get_window(window);
+    g_object_set_data(G_OBJECT(w), "virt-viewer-window", window);
+    gtk_application_add_window(GTK_APPLICATION(self), w);
 
     if (self->priv->fullscreen)
         app_window_try_fullscreen(self, window, nth);
 
-    w = virt_viewer_window_get_window(window);
     g_signal_connect(w, "hide", G_CALLBACK(viewer_window_visible_cb), self);
     g_signal_connect(w, "show", G_CALLBACK(viewer_window_visible_cb), self);
     g_signal_connect(w, "focus-in-event", G_CALLBACK(viewer_window_focus_in_cb), self);
@@ -1046,8 +986,6 @@ static void virt_viewer_app_remove_nth_window(VirtViewerApp *self,
     g_debug("Remove window %d %p", nth, win);
     self->priv->windows = g_list_remove(self->priv->windows, win);
 
-    g_signal_emit(self, signals[SIGNAL_WINDOW_REMOVED], 0, win);
-
     g_object_unref(win);
 }
 
@@ -1401,7 +1339,7 @@ virt_viewer_app_default_deactivated(VirtViewerApp *self, gboolean connect_error)
     }
 
     if (self->priv->quit_on_disconnect)
-        gtk_main_quit();
+        g_application_quit(G_APPLICATION(self));
 }
 
 static void
@@ -1479,7 +1417,7 @@ virt_viewer_app_disconnected(VirtViewerSession *session G_GNUC_UNUSED, const gch
         virt_viewer_app_hide_all_windows(self);
 
     if (priv->quitting)
-        gtk_main_quit();
+        g_application_quit(G_APPLICATION(self));
 
     if (connect_error) {
         GtkWidget *dialog = virt_viewer_app_make_message_dialog(self,
@@ -1701,6 +1639,7 @@ virt_viewer_app_dispose (GObject *object)
         g_hash_table_unref(tmp);
     }
 
+    priv->resource = NULL;
     g_clear_object(&priv->session);
     g_free(priv->title);
     priv->title = NULL;
@@ -1746,6 +1685,7 @@ gboolean virt_viewer_app_start(VirtViewerApp *self, GError **error)
 
 static int opt_zoom = NORMAL_ZOOM_LEVEL;
 static gchar *opt_hotkeys = NULL;
+static gboolean opt_version = FALSE;
 static gboolean opt_verbose = FALSE;
 static gboolean opt_debug = FALSE;
 static gboolean opt_fullscreen = FALSE;
@@ -1765,8 +1705,6 @@ virt_viewer_app_init(VirtViewerApp *self)
     self->priv = GET_PRIVATE(self);
 
     gtk_window_set_default_icon_name("virt-viewer");
-    virt_viewer_app_set_debug(opt_debug);
-    virt_viewer_app_set_fullscreen(self, opt_fullscreen);
 
     self->priv->displays = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, g_object_unref);
     self->priv->config = g_key_file_new();
@@ -1782,14 +1720,7 @@ virt_viewer_app_init(VirtViewerApp *self)
 
     g_clear_error(&error);
 
-    if (opt_zoom < MIN_ZOOM_LEVEL || opt_zoom > MAX_ZOOM_LEVEL) {
-        g_printerr(_("Zoom level must be within %d-%d\n"), MIN_ZOOM_LEVEL, MAX_ZOOM_LEVEL);
-        opt_zoom = NORMAL_ZOOM_LEVEL;
-    }
-
     self->priv->initial_display_map = virt_viewer_app_get_monitor_mapping_for_section(self, "fallback");
-    self->priv->verbose = opt_verbose;
-    self->priv->quit_on_disconnect = opt_kiosk ? opt_kiosk_quit : TRUE;
     g_signal_connect(self, "notify::guest-name", G_CALLBACK(title_maybe_changed), NULL);
     g_signal_connect(self, "notify::title", G_CALLBACK(title_maybe_changed), NULL);
     g_signal_connect(self, "notify::guri", G_CALLBACK(title_maybe_changed), NULL);
@@ -1831,7 +1762,7 @@ virt_viewer_update_smartcard_accels(VirtViewerApp *self)
         sw_smartcard = FALSE;
     }
     if (sw_smartcard) {
-        g_warning("enabling smartcard shortcuts");
+        g_debug("enabling smartcard shortcuts");
         gtk_accel_map_change_entry("<virt-viewer>/file/smartcard-insert",
                                    priv->insert_smartcard_accel_key,
                                    priv->insert_smartcard_accel_mods,
@@ -1841,16 +1772,27 @@ virt_viewer_update_smartcard_accels(VirtViewerApp *self)
                                    priv->remove_smartcard_accel_mods,
                                    TRUE);
     } else {
-        g_warning("disabling smartcard shortcuts");
+        g_debug("disabling smartcard shortcuts");
         gtk_accel_map_change_entry("<virt-viewer>/file/smartcard-insert", 0, 0, TRUE);
         gtk_accel_map_change_entry("<virt-viewer>/file/smartcard-remove", 0, 0, TRUE);
     }
 }
 
 static void
-virt_viewer_app_constructed(GObject *object)
+virt_viewer_app_on_application_startup(GApplication *app)
 {
-    VirtViewerApp *self = VIRT_VIEWER_APP(object);
+    VirtViewerApp *self = VIRT_VIEWER_APP(app);
+    GError *error = NULL;
+
+    G_APPLICATION_CLASS(virt_viewer_app_parent_class)->startup(app);
+
+    self->priv->resource = virt_viewer_get_resource();
+
+    virt_viewer_app_set_debug(opt_debug);
+    virt_viewer_app_set_fullscreen(self, opt_fullscreen);
+
+    self->priv->verbose = opt_verbose;
+    self->priv->quit_on_disconnect = opt_kiosk ? opt_kiosk_quit : TRUE;
 
     self->priv->main_window = virt_viewer_app_window_new(self,
                                                          virt_viewer_app_get_first_monitor(self));
@@ -1858,35 +1800,106 @@ virt_viewer_app_constructed(GObject *object)
 
     virt_viewer_app_set_kiosk(self, opt_kiosk);
     virt_viewer_app_set_hotkeys(self, opt_hotkeys);
+
+    if (opt_zoom < MIN_ZOOM_LEVEL || opt_zoom > MAX_ZOOM_LEVEL) {
+        g_printerr(_("Zoom level must be within %d-%d\n"), MIN_ZOOM_LEVEL, MAX_ZOOM_LEVEL);
+        opt_zoom = NORMAL_ZOOM_LEVEL;
+    }
+
     virt_viewer_window_set_zoom_level(self->priv->main_window, opt_zoom);
 
-    virt_viewer_set_insert_smartcard_accel(self, GDK_F8, GDK_SHIFT_MASK);
-    virt_viewer_set_remove_smartcard_accel(self, GDK_F9, GDK_SHIFT_MASK);
-    gtk_accel_map_add_entry("<virt-viewer>/view/toggle-fullscreen", GDK_F11, 0);
-    gtk_accel_map_add_entry("<virt-viewer>/view/release-cursor", GDK_F12, GDK_SHIFT_MASK);
-    gtk_accel_map_add_entry("<virt-viewer>/view/zoom-reset", GDK_0, GDK_CONTROL_MASK);
-    gtk_accel_map_add_entry("<virt-viewer>/view/zoom-out", GDK_minus, GDK_CONTROL_MASK);
-    gtk_accel_map_add_entry("<virt-viewer>/view/zoom-in", GDK_plus, GDK_CONTROL_MASK);
-    gtk_accel_map_add_entry("<virt-viewer>/send/secure-attention", GDK_End, GDK_CONTROL_MASK | GDK_MOD1_MASK);
+    virt_viewer_set_insert_smartcard_accel(self, GDK_KEY_F8, GDK_SHIFT_MASK);
+    virt_viewer_set_remove_smartcard_accel(self, GDK_KEY_F9, GDK_SHIFT_MASK);
+    gtk_accel_map_add_entry("<virt-viewer>/view/toggle-fullscreen", GDK_KEY_F11, 0);
+    gtk_accel_map_add_entry("<virt-viewer>/view/release-cursor", GDK_KEY_F12, GDK_SHIFT_MASK);
+    gtk_accel_map_add_entry("<virt-viewer>/view/zoom-reset", GDK_KEY_0, GDK_CONTROL_MASK);
+    gtk_accel_map_add_entry("<virt-viewer>/view/zoom-out", GDK_KEY_minus, GDK_CONTROL_MASK);
+    gtk_accel_map_add_entry("<virt-viewer>/view/zoom-in", GDK_KEY_plus, GDK_CONTROL_MASK);
+    gtk_accel_map_add_entry("<virt-viewer>/send/secure-attention", GDK_KEY_End, GDK_CONTROL_MASK | GDK_MOD1_MASK);
+
+    if (!virt_viewer_app_start(self, &error)) {
+        if (error && !g_error_matches(error, VIRT_VIEWER_ERROR, VIRT_VIEWER_ERROR_CANCELLED))
+            virt_viewer_app_simple_message_dialog(self, error->message);
+
+        g_clear_error(&error);
+        g_application_quit(app);
+        return;
+    }
+}
+
+static gboolean
+virt_viewer_app_local_command_line (GApplication   *gapp,
+                                    gchar        ***args,
+                                    int            *status)
+{
+    VirtViewerApp *self = VIRT_VIEWER_APP(gapp);
+    gboolean ret = FALSE;
+    gint argc = g_strv_length(*args);
+    GError *error = NULL;
+    GOptionContext *context = g_option_context_new(NULL);
+    GOptionGroup *group = g_option_group_new("virt-viewer", NULL, NULL, gapp, NULL);
+
+    *status = 0;
+    g_option_context_set_main_group(context, group);
+    VIRT_VIEWER_APP_GET_CLASS(self)->add_option_entries(self, context, group);
+
+    g_option_context_add_group(context, gtk_get_option_group(FALSE));
+
+#ifdef HAVE_GTK_VNC
+    g_option_context_add_group(context, vnc_display_get_option_group());
+#endif
+
+#ifdef HAVE_SPICE_GTK
+    g_option_context_add_group(context, spice_get_option_group());
+#endif
+
+    if (!g_option_context_parse(context, &argc, args, &error)) {
+        if (error != NULL) {
+            g_printerr(_("%s\n"), error->message);
+            g_error_free(error);
+        }
+
+        *status = 1;
+        ret = TRUE;
+        goto end;
+    }
+
+    if (opt_version) {
+        g_print(_("%s version %s"), g_get_prgname(), VERSION BUILDID);
+#ifdef REMOTE_VIEWER_OS_ID
+        g_print(" (OS ID: %s)", REMOTE_VIEWER_OS_ID);
+#endif
+        g_print("\n");
+        ret = TRUE;
+    }
+
+end:
+    g_option_context_free(context);
+    return ret;
 }
 
 static void
 virt_viewer_app_class_init (VirtViewerAppClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
+    GApplicationClass *g_app_class = G_APPLICATION_CLASS(klass);
 
     g_type_class_add_private (klass, sizeof (VirtViewerAppPrivate));
 
-    object_class->constructed = virt_viewer_app_constructed;
     object_class->get_property = virt_viewer_app_get_property;
     object_class->set_property = virt_viewer_app_set_property;
     object_class->dispose = virt_viewer_app_dispose;
 
+    g_app_class->local_command_line = virt_viewer_app_local_command_line;
+    g_app_class->startup = virt_viewer_app_on_application_startup;
+    g_app_class->command_line = NULL; /* inhibit GApplication default handler */
+
     klass->start = virt_viewer_app_default_start;
     klass->initial_connect = virt_viewer_app_default_initial_connect;
     klass->activate = virt_viewer_app_default_activate;
     klass->deactivated = virt_viewer_app_default_deactivated;
     klass->open_connection = virt_viewer_app_default_open_connection;
+    klass->add_option_entries = virt_viewer_app_add_option_entries;
 
     g_object_class_install_property(object_class,
                                     PROP_VERBOSE,
@@ -1992,28 +2005,6 @@ virt_viewer_app_class_init (VirtViewerAppClass *klass)
                                                         G_PARAM_READABLE |
                                                         G_PARAM_WRITABLE |
                                                         G_PARAM_STATIC_STRINGS));
-
-    signals[SIGNAL_WINDOW_ADDED] =
-        g_signal_new("window-added",
-                     G_OBJECT_CLASS_TYPE(object_class),
-                     G_SIGNAL_RUN_LAST,
-                     G_STRUCT_OFFSET(VirtViewerAppClass, window_added),
-                     NULL, NULL,
-                     g_cclosure_marshal_VOID__OBJECT,
-                     G_TYPE_NONE,
-                     1,
-                     G_TYPE_OBJECT);
-
-    signals[SIGNAL_WINDOW_REMOVED] =
-        g_signal_new("window-removed",
-                     G_OBJECT_CLASS_TYPE(object_class),
-                     G_SIGNAL_RUN_LAST,
-                     G_STRUCT_OFFSET(VirtViewerAppClass, window_removed),
-                     NULL, NULL,
-                     g_cclosure_marshal_VOID__OBJECT,
-                     G_TYPE_NONE,
-                     1,
-                     G_TYPE_OBJECT);
 }
 
 void
@@ -2072,18 +2063,23 @@ virt_viewer_app_set_hotkeys(VirtViewerApp *self, const gchar *hotkeys_str)
 
     for (hotkey = hotkeys; *hotkey != NULL; hotkey++) {
         gchar *key = strstr(*hotkey, "=");
-        if (key == NULL) {
-            g_warn_if_reached();
+        const gchar *value = (key == NULL) ? NULL : (*key = '\0', key + 1);
+        if (value == NULL || *value == '\0') {
+            g_warning("missing value for key '%s'", *hotkey);
             continue;
         }
-        *key = '\0';
 
-        gchar *accel = spice_hotkey_to_gtk_accelerator(key + 1);
+        gchar *accel = spice_hotkey_to_gtk_accelerator(value);
         guint accel_key;
         GdkModifierType accel_mods;
         gtk_accelerator_parse(accel, &accel_key, &accel_mods);
         g_free(accel);
 
+        if (accel_key == 0 && accel_mods == 0) {
+            g_warning("Invalid value '%s' for key '%s'", value, *hotkey);
+            continue;
+        }
+
         if (g_str_equal(*hotkey, "toggle-fullscreen")) {
             gtk_accel_map_change_entry("<virt-viewer>/view/toggle-fullscreen", accel_key, accel_mods, TRUE);
         } else if (g_str_equal(*hotkey, "release-cursor")) {
@@ -2293,8 +2289,8 @@ window_update_menu_displays_cb(gpointer value,
         gboolean visible;
         gchar *label;
 
-        label = g_strdup_printf(_("Display %d"), nth + 1);
-        item = gtk_check_menu_item_new_with_label(label);
+        label = g_strdup_printf(_("Display _%d"), nth + 1);
+        item = gtk_check_menu_item_new_with_mnemonic(label);
         g_free(label);
 
         visible = vwin && gtk_widget_get_visible(GTK_WIDGET(virt_viewer_window_get_window(vwin)));
@@ -2469,7 +2465,7 @@ static GtkWidget *
 virt_viewer_app_get_preferences(VirtViewerApp *self)
 {
     VirtViewerSession *session = virt_viewer_app_get_session(self);
-    GtkBuilder *builder = virt_viewer_util_load_ui("virt-viewer-preferences.xml");
+    GtkBuilder *builder = virt_viewer_util_load_ui("virt-viewer-preferences.ui");
     gboolean can_share_folder = virt_viewer_session_can_share_folder(session);
     GtkWidget *preferences = self->priv->preferences;
     gchar *path;
@@ -2553,10 +2549,14 @@ option_kiosk_quit(G_GNUC_UNUSED const gchar *option_name,
     return FALSE;
 }
 
-GOptionGroup*
-virt_viewer_app_get_option_group(void)
+static void
+virt_viewer_app_add_option_entries(G_GNUC_UNUSED VirtViewerApp *self,
+                                   G_GNUC_UNUSED GOptionContext *context,
+                                   GOptionGroup *group)
 {
     static const GOptionEntry options [] = {
+        { "version", 'V', 0, G_OPTION_ARG_NONE, &opt_version,
+          N_("Display version information"), NULL },
         { "zoom", 'z', 0, G_OPTION_ARG_INT, &opt_zoom,
           N_("Zoom level of window, in percentage"), "ZOOM" },
         { "full-screen", 'f', 0, G_OPTION_ARG_NONE, &opt_fullscreen,
@@ -2573,11 +2573,8 @@ virt_viewer_app_get_option_group(void)
           N_("Display debugging information"), NULL },
         { NULL, 0, 0, G_OPTION_ARG_NONE, NULL, NULL, NULL }
     };
-    GOptionGroup *group;
-    group = g_option_group_new("virt-viewer", NULL, NULL, NULL, NULL);
-    g_option_group_add_entries(group, options);
 
-    return group;
+    g_option_group_add_entries(group, options);
 }
 
 gboolean virt_viewer_app_get_session_cancelled(VirtViewerApp *self)
